<?php
/**
 * @package  eWay Rapid Joomla! HikaShop
 * @see      https://github.com/eWAYPayment/eway-rapid-php
 */
?><?php

namespace Eway\Rapid\Model\Support;

use Eway\Rapid\Contract\Arrayable;
use Eway\Rapid\Exception\MassAssignmentException;

trait HasAttributesTrait
{
	protected static $mutatorCache = [];

	protected $attributes = [];

	protected $fillable = [];

	public function fill(array $attributes)
	{
		$class = get_called_class();

		foreach ($attributes as $key => $value) {
			if ($this->isFillable($key)) {
				$this->setAttribute($key, $value);
			} else {
				throw new MassAssignmentException(sprintf("Unexpected attribute '%s' for class %s", $key, $class));
			}
		}

		return $this;
	}

	public function isFillable($key)
	{
		if (in_array($key, $this->fillable)) {
			return true;
		}

		return empty($this->fillable);
	}

	public function getAttribute($key)
	{
		$value = $this->getAttributeFromArray($key);

		return $value;
	}

	public function setAttribute($key, $value)
	{
		if ($this->hasSetMutator($key)) {
			$method = 'set'.$this->getStudlyCase($key).'Attribute';

			return $this->{$method}($value);
		}

		$this->attributes[$key] = $value;

		return $this;
	}

	public function hasGetMutator($key)
	{
		return method_exists($this, 'get'.$this->getStudlyCase($key).'Attribute');
	}

	public function hasSetMutator($key)
	{
		return method_exists($this, 'set'.$this->getStudlyCase($key).'Attribute');
	}

	public function attributesToArray()
	{
		$attributes = $this->attributes;

		foreach ($attributes as $key => $value) {
			$attributes[$key] = $this->_toArrayRecursive($value);
		}

		return $attributes;
	}

	public function __get($key)
	{
		return $this->getAttribute($key);
	}

	public function __set($key, $value)
	{
		$this->setAttribute($key, $value);
	}

	public function __isset($key)
	{
		return isset($this->attributes[$key]) || ($this->hasGetMutator($key) && !is_null($this->getAttribute($key)));
	}

	public function __unset($key)
	{
		unset($this->attributes[$key]);
	}

	protected function getAttributeFromArray($key)
	{
		if (array_key_exists($key, $this->attributes)) {
			return $this->attributes[$key];
		}

		trigger_error(sprintf("Undefined property '%s' in class '%s'", $key, get_called_class()), E_USER_NOTICE);

		return null;
	}

	protected function getStudlyCase($str)
	{
		$str = ucwords(str_replace(['-', '_'], ' ', $str));

		return str_replace(' ', '', $str);
	}

	private function _toArrayRecursive($subject)
	{
		if (is_array($subject)) {
			foreach ($subject as $key => $value) {
				$subject[$key] = $this->_toArrayRecursive($value);
			}

			return $subject;
		} else {
			return $subject instanceof Arrayable ? $subject->toArray() : $subject;
		}
	}
}
