<?php
/**
 * @package  eWay Rapid Joomla! HikaShop
 * @see      https://github.com/eWAYPayment/eway-rapid-php
 */
?><?php

namespace Eway\Rapid\Service;

use Eway\Rapid\Contract\Client;
use Eway\Rapid\Contract\Http\ResponseInterface;
use Eway\Rapid\Contract\HttpService as HttpServiceContract;
use Eway\Rapid\Service\Http\Response;
use InvalidArgumentException;

class Http implements HttpServiceContract
{
	private $key;

	private $password;

	private $baseUrl;

	public function __construct($key = null, $password = null, $baseUrl = null)
	{
		$this->key = $key;
		$this->password = $password;
		$this->baseUrl = $baseUrl;
	}

	public function getTransaction($reference)
	{
		return $this->_getRequest([self::API_TRANSACTION_QUERY, ['Reference' => $reference]]);
	}

	public function getTransactionInvoiceNumber($invoiceNumber)
	{
		return $this->_getRequest([self::API_TRANSACTION_INVOICE_NUMBER_QUERY, ['InvoiceNumber' => $invoiceNumber]]);
	}

	public function getTransactionInvoiceReference($invoiceReference)
	{
		return $this->_getRequest([self::API_TRANSACTION_INVOICE_REFERENCE_QUERY, ['InvoiceReference' => $invoiceReference]]);
	}

	public function postTransaction($data)
	{
		return $this->_postRequest(self::API_TRANSACTION, $data);
	}

	public function postTransactionRefund($transactionId, $data)
	{
		return $this->_postRequest([self::API_TRANSACTION_REFUND, ['TransactionID' => $transactionId]], $data);
	}

	public function postAccessCodeShared($data)
	{
		return $this->_postRequest(self::API_ACCESS_CODE_SHARED, $data);
	}

	public function getAccessCode($accessCode)
	{
		return $this->_getRequest([self::API_ACCESS_CODE_QUERY, ['AccessCode' => $accessCode]]);
	}

	public function postAccessCode($data)
	{
		return $this->_postRequest(self::API_ACCESS_CODE, $data);
	}

	public function getCustomer($tokenCustomerId)
	{
		return $this->_getRequest([self::API_CUSTOMER_QUERY, ['TokenCustomerID' => $tokenCustomerId]]);
	}

	public function postCapturePayment($data)
	{
		return $this->_postRequest(self::API_CAPTURE_PAYMENT, $data);
	}

	public function postCancelAuthorisation($data)
	{
		return $this->_postRequest(self::API_CANCEL_AUTHORISATION, $data);
	}

	public function getUri($uri, $withBaseUrl = true)
	{
		$baseUrl = $withBaseUrl ? $this->baseUrl : '';
		if (!is_array($uri)) {
			return $baseUrl.$uri;
		}
		list($uri, $vars) = $uri;
		$uri = $baseUrl.$uri;
		if (!is_array($vars)) {
			throw new InvalidArgumentException();
		}
		foreach ($vars as $key => $value) {
			$uri = str_replace('{'.$key.'}', $value, $uri);
		}

		return $uri;
	}

	public function setKey($key)
	{
		$this->key = $key;

		return $this;
	}

	public function getKey()
	{
		return $this->key;
	}

	public function setPassword($password)
	{
		$this->password = $password;

		return $this;
	}

	public function getPassword()
	{
		return $this->password;
	}

	public function setBaseUrl($baseUrl)
	{
		$this->baseUrl = $baseUrl;

		return $this;
	}

	public function getBaseUrl()
	{
		return $this->baseUrl;
	}

	private function _getRequest($url)
	{
		return $this->_request('GET', $url);
	}

	private function _postRequest($url, $data)
	{
		return $this->_request('POST', $url, $data);
	}

	private function _request($method, $uri, $data = [])
	{
		$uri = $this->getUri($uri);

		$headers = [];

		$credentials = $this->key.':'.$this->password;

		$agent = sprintf("%s %s", Client::NAME, Client::VERSION);

		$ch = curl_init();

		$options = [
			CURLOPT_URL => $uri,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_HEADER => true,
			CURLOPT_USERAGENT => $agent,
			CURLOPT_USERPWD => $credentials,
			CURLOPT_TIMEOUT => 60,
			CURLOPT_SSL_VERIFYPEER => false,
		];

		if (strtoupper($method) === 'POST') {
			$jsonData = json_encode($data);

			$headers[] = 'Content-Type: application/json';
			$headers[] = 'Content-Length: '.strlen($jsonData);

			$options[CURLOPT_CUSTOMREQUEST] = 'POST';
			$options[CURLOPT_POSTFIELDS] = $jsonData;
		}

		$options[CURLOPT_HTTPHEADER] = $headers;

		curl_setopt_array($ch, $options);

		$rawResponse = curl_exec($ch);

		$statusCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		$headerSize = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
		$responseBody = substr($rawResponse, $headerSize);

		if ($rawResponse === false) {
		   $responseBody = curl_error($ch);
		}

		$response = new Response($statusCode, $responseBody);

		curl_close($ch);

		return $response;
	}
}
